import 'package:flutter/material.dart';

class CacheItemSubscriber {
  final int id;
  final VoidCallback notify;

  CacheItemSubscriber({
    required this.id,
    required this.notify,
  });

  @override
  String toString() {
    return 'Subscriber($id)';
  }
}

class CacheItem<T> {
  List<CacheItemSubscriber> subscribers;
  String key;
  T? data;

  CacheItem({
    required this.subscribers,
    required this.key,
  });

  @override
  String toString() {
    String str = 'CacheItem($key)-[';
    str += subscribers.map((e) => e.toString()).join(",");
    // str += data.toString();
    return str + ']';
  }
}

class _Cache {
  Map<String, CacheItem> _cache = {};

  bool contains(String key) {
    return _cache.containsKey(key);
  }

  void addSubscriber(String key, CacheItemSubscriber subscriber) {
    if (_cache.containsKey(key)) {
      _cache[key]!.subscribers.add(subscriber);
    } else {
      _cache[key] = CacheItem(key: key, subscribers: [subscriber]);
    }
    print(this.toString());
  }

  List<CacheItemSubscriber> getSubscribers(String key) {
    if (!_cache.containsKey(key)) {
      return [];
    }
    return _cache[key]!.subscribers;
  }

  void notifySubscribers(String key) {
    for (var subscriber in this.getSubscribers(key)) {
      subscriber.notify();
    }
  }

  void removeSubscribers(String key) {
    if (_cache.containsKey(key)) {
      _cache[key]!.subscribers = [];
    }
  }

  void removeSubscriber(String key, int subscriberId) {
    if (_cache.containsKey(key)) {
      _cache[key]!.subscribers.removeWhere(
            (element) => element.id == subscriberId,
          );
    }
    print(this.toString());
  }

  void addData<T>(String key, T t) {
    if (_cache.containsKey(key)) {
      _cache[key]!.data = t;
    }
  }

  T? getData<T>(String key) {
    if (_cache.containsKey(key)) {
      return _cache[key]!.data;
    }
    return null;
  }

  void reset() {
    _cache = {};
  }

  @override
  String toString() {
    String str = '-------------------------------------------\n';
    str += _cache.entries.map((e) => e.value.toString()).join("\n");
    str += '\n-------------------------------------------';
    return str;
  }
}

var cache = _Cache();
